package dk.camelot64.kickc.fragment;

import dk.camelot64.kickc.fragment.signature.AsmFragmentSignature;
import dk.camelot64.kickc.model.TargetCpu;
import dk.camelot64.kickc.parser.KickCParser;

/**
 * An ASM fragment template usable for generating ASM code for different bindings.
 */
public interface AsmFragmentTemplate {

    /**
     * Get the signature of the ASM fragment template.
     * The signature describes the operation performed and the types of the parameters.
     *
     * @return The signature
     */
    AsmFragmentSignature getSignature();

    /**
     * Get the target CPU of the ASM.
     *
     * @return The target CPU
     */
    TargetCpu getTargetCpu();
    
    /**
     * Get the ASM body as parsed ASM.
     * The ASM body contains parameter placeholders where bindings will be used to place actual values.
     *
     * @return The ASM body
     */
    KickCParser.AsmLinesContext getBodyAsm();

    /**
     * Get the registers clobbered by the ASM fragment
     *
     * @return The clobber
     */
    AsmFragmentClobber getClobber();

    /**
     * Get the cycle cost of the ASM.
     *
     * @return The cycle cost
     */
    double getCycles();

}
