package dk.camelot64.kickc.fragment;

import dk.camelot64.kickc.fragment.signature.AsmFragmentSignature;
import dk.camelot64.kickc.model.TargetCpu;

import java.util.Collection;

/**
 * Abstract factory for creating ASM fragment templates from signatures.
 */
public interface AsmFragmentTemplateFactory {

    /**
     * Get the CPU that the factory produces templates for
     *
     * @return The target CPU
     */
    TargetCpu getCpu();

    /**
     * Get ASM templates for a signature.
     * <p>
     * Returns zero to many templates that can generate ASM for the signature.
     * If many templates are returned they will all generate valid ASM, but may have different characteristics such as clobber and cycles.
     * Returns <code>null</code> or zero templates if the factory cannot find a fragment template for the signature.
     *
     * @param signature The signature describing the ASM fragment
     * @return ASM fragment templates that can be used for generating ASM
     */
    Collection<AsmFragmentTemplate> getTemplates(AsmFragmentSignature signature);

}
