package dk.camelot64.kickc.fragment;

import dk.camelot64.kickc.parser.KickCParser;
import dk.camelot64.kickc.parser.KickCParserBaseVisitor;
import org.antlr.v4.runtime.tree.TerminalNode;

import java.util.List;

/**
 * Prints ASM template body as string.
 */
public class AsmTemplateBodyPrinter extends KickCParserBaseVisitor {

    private final StringBuffer out;

    private AsmTemplateBodyPrinter(StringBuffer out) {
        this.out = out;
    }

    public static String print(KickCParser.AsmLinesContext asmBody) {
        final StringBuffer stringBuffer = new StringBuffer();
        final AsmTemplateBodyPrinter bodyPrinter = new AsmTemplateBodyPrinter(stringBuffer);
        bodyPrinter.visit(asmBody);
        return stringBuffer.toString();
    }

    private void printTags(List<TerminalNode> tags) {
        tags.forEach(tag -> out.append(" ").append(tag));
    }

    @Override
    public Object visitAsmLabelName(KickCParser.AsmLabelNameContext ctx) {
        out.append(ctx.ASM_NAME()).append(ctx.ASM_COLON());
        final List<TerminalNode> tags = ctx.ASM_TAG();
        printTags(tags);
        out.append("\n");
        return null;
    }


    @Override
    public Object visitAsmLabelReplace(KickCParser.AsmLabelReplaceContext ctx) {
        out.append(ctx.ASM_CURLY_BEGIN());
        out.append(ctx.ASM_NAME());
        out.append(ctx.ASM_CURLY_END());
        out.append(ctx.ASM_COLON());
        printTags(ctx.ASM_TAG());
        out.append("\n");
        return null;
    }

    @Override
    public Object visitAsmLabelMulti(KickCParser.AsmLabelMultiContext ctx) {
        out.append(ctx.ASM_MULTI_NAME());
        out.append(ctx.ASM_COLON());
        printTags(ctx.ASM_TAG());
        out.append("\n");
        return null;
    }

    @Override
    public Object visitAsmBytes(KickCParser.AsmBytesContext ctx) {
        printTags(ctx.ASM_TAG());
        return null;
    }

    @Override
    public Object visitAsmInstruction(KickCParser.AsmInstructionContext ctx) {
        out.append(ctx.ASM_MNEMONIC());
        if (ctx.asmParamMode() != null) {
            out.append(" ");
            this.visit(ctx.asmParamMode());
        }
        printTags(ctx.ASM_TAG());
        out.append("\n");
        return null;
    }

    @Override
    public Object visitAsmModeImm(KickCParser.AsmModeImmContext ctx) {
        out.append(ctx.ASM_IMM());
        this.visit(ctx.asmExpr());
        return null;
    }

    @Override
    public Object visitAsmModeAbs(KickCParser.AsmModeAbsContext ctx) {
        this.visit(ctx.asmExpr());
        return null;
    }

    @Override
    public Object visitAsmModeAbs2(KickCParser.AsmModeAbs2Context ctx) {
        this.visit(ctx.asmExpr(0));
        out.append(ctx.ASM_COMMA());
        this.visit(ctx.asmExpr(1));
        return null;
    }

    @Override
    public Object visitAsmModeInd(KickCParser.AsmModeIndContext ctx) {
        out.append(ctx.ASM_PAR_BEGIN());
        this.visit(ctx.asmExpr());
        out.append(ctx.ASM_PAR_END());
        return null;
    }

    @Override
    public Object visitAsmModeIndIdxXY(KickCParser.AsmModeIndIdxXYContext ctx) {
        out.append(ctx.ASM_PAR_BEGIN());
        this.visit(ctx.asmExpr());
        out.append(ctx.ASM_PAR_END());
        out.append(ctx.ASM_COMMA());
        out.append(ctx.ASM_NAME());
        return null;
    }

    @Override
    public Object visitAsmExprPar(KickCParser.AsmExprParContext ctx) {
        out.append(ctx.ASM_BRACKET_BEGIN());
        this.visit(ctx.asmExpr());
        out.append(ctx.ASM_BRACKET_END());
        return null;
    }

    @Override
    public Object visitAsmExprInt(KickCParser.AsmExprIntContext ctx) {
        out.append(ctx.ASM_NUMBER());
        return null;
    }

    @Override
    public Object visitAsmExprLabel(KickCParser.AsmExprLabelContext ctx) {
        out.append(ctx.ASM_NAME());
        return null;
    }

    @Override
    public Object visitAsmExprLabelRel(KickCParser.AsmExprLabelRelContext ctx) {
        out.append(ctx.ASM_MULTI_REL());
        out.append(" ");
        return null;
    }

    @Override
    public Object visitAsmExprUnary(KickCParser.AsmExprUnaryContext ctx) {
        out.append((TerminalNode) ctx.getChild(0));
        this.visit(ctx.asmExpr());
        return null;
    }

    @Override
    public Object visitAsmExprBinary(KickCParser.AsmExprBinaryContext ctx) {
        this.visit(ctx.asmExpr(0));
        out.append((TerminalNode) ctx.getChild(1));
        this.visit(ctx.asmExpr(1));
        return null;
    }

    @Override
    public Object visitAsmExprReplace(KickCParser.AsmExprReplaceContext ctx) {
        out.append(ctx.ASM_CURLY_BEGIN());
        out.append(ctx.ASM_NAME());
        out.append(ctx.ASM_CURLY_END());
        return null;
    }
}
