package dk.camelot64.kickc.fragment.synthesis;

import dk.camelot64.kickc.fragment.AsmFragmentTemplate;
import dk.camelot64.kickc.fragment.signature.AsmFragmentSignature;

import java.util.List;

/**
 * A synthesis rule can create new {@link AsmFragmentTemplate}s from other {@link AsmFragmentTemplate}s.
 */
public interface AsmFragmentTemplateSynthesisRule {

    /**
     * Is the rule match usable for synthesizing a fragment template
     *
     * @param signature The fragment template signature
     * @return true if the rule matches the signature
     */
    boolean matches(AsmFragmentSignature signature);

    /**
     * The signatures of the sub-templates that are needed to generate an ASM fragment template
     *
     * @param signature The signature of the fragment to synthesize
     * @return Signatures of the sub-templates that are needed to synthesize the fragment template.
     */
    List<AsmFragmentSignature> getSubSignatures(AsmFragmentSignature signature);

    /**
     * Synthesize a fragment template from a sub fragment template.
     *
     * @param signature   The signature to synthesize
     * @param subTemplates The sub-templates that matches the sub-signatures
     * @return The synthesized ASM fragment template.
     * Null if the fragment cannot be synthesized (for instance due to clobber constraints).
     */
    AsmFragmentTemplate synthesize(AsmFragmentSignature signature, List<AsmFragmentTemplate> subTemplates);
}
